@@ -10,11 +10,11 @@
 
 import argparse
 import dataclasses
-import distutils.spawn
 import importlib
 import inspect
 import os
 import os.path
+import shutil
 import sys
 import textwrap
 from abc import ABC, abstractmethod
@@ -85,6 +85,8 @@ def _node_repr_recursive(  # noqa: C901
 
             def _get_default(fld: "dataclasses.Field[object]") -> object:
                 if fld.default_factory is not dataclasses.MISSING:
+                    # pyre-fixme[29]: `Union[dataclasses._MISSING_TYPE,
+                    #  dataclasses._DefaultFactory[object]]` is not a function.
                     return fld.default_factory()
                 return fld.default
 
@@ -373,10 +375,7 @@ def _find_and_load_config(proc_name: str) -> Dict[str, Any]:
 
     # Make sure that the formatter is findable.
     if config["formatter"]:
-        exe = (
-            distutils.spawn.find_executable(config["formatter"][0])
-            or config["formatter"][0]
-        )
+        exe = shutil.which(config["formatter"][0]) or config["formatter"][0]
         config["formatter"] = [os.path.abspath(exe), *config["formatter"][1:]]
 
     return config
@@ -783,6 +782,8 @@ def _list_impl(proc_name: str, command_args: List[str]) -> int:  # noqa: C901
             continue
 
         # Grab the path, try to import all of the files inside of it.
+        # pyre-fixme[6]: For 1st argument expected `PathLike[Variable[AnyStr <:
+        #  [str, bytes]]]` but got `Optional[str]`.
         path = os.path.dirname(os.path.abspath(imported_module.__file__))
         for name, imported_module in _recursive_find(path, module):
             for objname in dir(imported_module):
